from enum import Enum, IntEnum

CAM_MAGIC = 0xf1


class PacketType(Enum):
    Close = 0xf0
    LanSearchExt = 0x32
    LanSearch = 0x30
    P2PAlive = 0xe0
    P2PAliveAck = 0xe1
    Hello = 0x00
    P2pRdy = 0x42
    P2pReq = 0x20
    LstReq = 0x67
    DrwAck = 0xd1
    Drw = 0xd0

    # From CSession_CtrlPkt_Proc incomplete
    PunchTo = 0x40
    PunchPkt = 0x41
    HelloAck = 0x01
    RlyTo = 0x02
    DevLgnAck = 0x11
    P2PReqAck = 0x21
    ListenReqAck = 0x69
    RlyHelloAck = 0x70  # always
    RlyHelloAck2 = 0x71  # if len >1??


class BinaryCommands(Enum):
    CFGID_VERSION = 0x0000
    CMD_ACK_OK = 0x0000
    CFGID_LANGUAGE = 0x0001
    CMD_ACK_UNAUTH = 0x0001
    CFGID_PRODUCTE = 0x0002
    CMD_ACK_NO_PRIVILEGE = 0x0002
    CFGID_UPGRADE = 0x0003
    CMD_ACK_INVALID_PARAM = 0x0003
    CFGID_P2P = 0x0004
    CMD_ACK_CMDEXCUTE_FAILED = 0x0004
    CFGID_TZ = 0x0005
    CMD_ACK_NONE_RESULT = 0x0005
    CFGID_USER = 0x0006
    CMD_ACK_UNKNOWN = 0x0006
    CFGID_OPR = 0x0007
    CFGID_SERIAL = 0x0008
    CFGID_WIRED = 0x0009
    CFGID_WLAN = 0x000A
    CFGID_OSD = 0x000B
    CFGID_IMG = 0x000C
    CFGID_CMOS = 0x000D
    CFGID_PTZ = 0x000E
    CFGID_AUDIO = 0x000F
    CFGID_VIDEO = 0x0010
    CFGID_RECPOLICY = 0x0011
    CFGID_RESCH = 0x0012
    CFGID_MDALARM = 0x0013
    CFGID_ADCALARM = 0x0014
    CFGID_INPUTALARM = 0x0015
    CFGID_SMTP = 0x0016
    CFGID_FTP = 0x0017
    CFGID_PUSH = 0x0018
    CFGID_WLANPMK = 0x0019
    CGICMD = 0x00FE
    BINCMD = 0x00FF
    CMD_ACK_ILLIGAL = 0x03E8
    CMD_DEV_BROADCAST = 0x0EFF
    CMD_SYSTEM_DFTCFG_IMPORT = 0x1000
    CMD_SYSTEM_DFTCFG_EXPORT = 0x1001
    CMD_SYSTEM_DFTCFG_RECOVERY = 0x1002
    CMD_SYSTEM_ITEMDFTCFG_RECOVERY = 0x1003
    CMD_SYSTEM_CRNCFG_EXPORT = 0x1004
    CMD_SYSTEM_CRNCFG_IMPORT = 0x1005
    CMD_SYSTEM_DFTCFG_CREATE = 0x1006
    CMD_SYSTEM_UPGRAD_SET = 0x1007
    CMD_SYSTEM_STATUS_GET = 0x1008
    CMD_SYSTEM_UPGRAD_GET = 0x1009
    CMD_SYSTEM_SHUTDOWN = 0x1010
    CMD_SYSTEM_REBOOT = 0x1011
    CMD_SYSTEM_INF_GET = 0x1012
    CMD_SYSTEM_ALIAS_SET = 0x1013
    CMD_SYSTEM_USER_CHK = 0x1020
    CMD_SYSTEM_USER_SET = 0x1021
    CMD_SYSTEM_USER_GET = 0x1022
    CMD_SYSTEM_USER_CHG = 0x1023
    CMD_SYSTEM_P2PPARAM_SET = 0x1030
    CMD_SYSTEM_OPRPOLICY_SET = 0x1031
    CMD_SYSTEM_OPRPOLICY_GET = 0x1032
    CMD_SYSTEM_P2PPARAM_GET = 0x1033
    CMD_SYSTEM_DATETIME_SET = 0x1040
    CMD_SYSTEM_DATETIME_GET = 0x1041
    CMD_NOTIFICATION = 0x1051
    ACK_SYSTEM_DFTCFG_IMPORT = 0x1100
    ACK_SYSTEM_DFTCFG_EXPORT = 0x1101
    ACK_SYSTEM_DFTCFG_RECOVERY = 0x1102
    ACK_SYSTEM_ITEMDFTCFG_RECOVERY = 0x1103
    ACK_SYSTEM_CRNCFG_EXPORT = 0x1104
    ACK_SYSTEM_CRNCFG_IMPORT = 0x1105
    ACK_SYSTEM_DFTCFG_CREATE = 0x1106
    ACK_SYSTEM_UPGRAD_SET = 0x1107
    ACK_SYSTEM_STATUS_GET = 0x1108
    ACK_SYSTEM_UPGRAD_GET = 0x1109
    ACK_SYSTEM_SHUTDOWN = 0x1110
    ACK_SYSTEM_REBOOT = 0x1111
    ACK_SYSTEM_INF_GET = 0x1112
    ACK_SYSTEM_ALIAS_SET = 0x1113
    ACK_SYSTEM_USER_CHK = 0x1120
    ACK_SYSTEM_USER_SET = 0x1121
    ACK_SYSTEM_USER_GET = 0x1122
    ACK_SYSTEM_USER_CHG = 0x1123
    ACK_SYSTEM_P2PPARAM_SET = 0x1130
    ACK_SYSTEM_OPRPOLICY_SET = 0x1131
    ACK_SYSTEM_OPRPOLICY_GET = 0x1132
    ACK_SYSTEM_P2PPARAM_GET = 0x1133
    ACK_SYSTEM_DATETIME_SET = 0x1140
    ACK_SYSTEM_DATETIME_GET = 0x1141
    ACK_NOTIFICATION = 0x1151
    CMD_SD_FORMAT = 0x2000
    CMD_SD_RECPOLICY_SET = 0x2001
    CMD_SD_RECPOLICY_GET = 0x2002
    CMD_SD_RECORDING_NOW = 0x2003
    CMD_SD_INFO_GET = 0x2004
    CMD_SD_RECORDFILE_GET = 0x2005
    CMD_SD_RECORDSCH_GET = 0x2006
    CMD_SD_RECORDSCH_SET = 0x2007
    CMD_SD_RETRIVEL = 0x2008
    CMD_SD_PICFILE_GET = 0x2009
    CMD_SD_PIC_CAPTURE = 0x200A
    CMD_SD_REC_DEL = 0x200B
    CMD_SD_PIC_DEL = 0x200C
    CMD_SD_SPL_DEL = 0x200D
    ACK_SD_FORMAT = 0x2100
    ACK_SD_RECPOLICY_SET = 0x2101
    ACK_SD_RECPOLICY_GET = 0x2102
    ACK_SD_RECORDING_NOW = 0x2103
    ACK_SD_INFO_GET = 0x2104
    ACK_SD_RECORDFILE_GET = 0x2105
    ACK_SD_RECORDSCH_GET = 0x2106
    ACK_SD_RECORDSCH_SET = 0x2107
    ACK_SD_RETRIVEL = 0x2108
    ACK_SD_PICFILE_GET = 0x2109
    ACK_SD_PIC_CAPTURE = 0x210A
    ACK_SD_REC_DEL = 0x210B
    ACK_SD_PIC_DEL = 0x210C
    ACK_SD_SPL_DEL = 0x210D
    CMD_PEER_LIVEAUDIO_START = 0x3000
    CMD_PEER_LIVEAUDIO_STOP = 0x3001
    CMD_LOCAL_LIVEAUDIO_START = 0x3002
    CMD_LOCAL_LIVEAUDIO_STOP = 0x3003
    CMD_PEER_AUDIOPARAM_SET = 0x3004
    CMD_PEER_AUDIOPARAM_GET = 0x3005
    CMD_PEER_AUDIOFILE_STARTPLAY = 0x3006
    CMD_PEER_AUDIOFILE_STOPPLAY = 0x3007
    CMD_PEER_AUDIOFILELIST_GET = 0x3008
    CMD_PEER_IRCUT_ONOFF = 0x300A
    CMD_PEER_LIGHTFILL_ONOFF = 0x300B
    CMD_PEER_LIVEVIDEO_START = 0x3010
    CMD_PEER_LIVEVIDEO_STOP = 0x3011
    CMD_PEER_PLAYBACK_START = 0x3012
    CMD_PEER_PLAYBACK_STOP = 0x3013
    CMD_PEER_PLAYBACK_SEEK = 0x3014
    CMD_PEER_PLAYBACK_SPEED = 0x3015
    CMD_PEER_PLAYBACK_PAUSE = 0x3016
    CMD_PEER_PLAYBACK_RESUME = 0x3017
    CMD_PEER_VIDEOPARAM_SET = 0x3018
    CMD_PEER_VIDEOPARAM_GET = 0x3019
    CMD_SNAPSHOT_GET = 0x301A
    CMD_PEER_PLAYBACK_END = 0x301B
    CMD_PEER_PLAYBACK_STEP = 0x301C
    CMD_DOORBELL_CALL_OPEN = 0x3020
    CMD_DOORBELL_CALL_CLOSE = 0x3021
    CMD_DOORBELL_CALL_ACCEPT = 0x3022
    CMD_DOORBELL_CALL_REJECT = 0x3023
    CMD_LOCAL_LIVEVIDIO_SEND_ON = 0x3024
    CMD_LOCAL_LIVEVIDIO_SEND_OFF = 0x3025
    CMD_LOCAL_AUDIO_STATUS_SET = 0x3026
    CMD_LOCAL_AUDIO_STATUS_GET = 0x3027
    CMD_LOCAL_AVREC_START = 0x3028
    CMD_LOCAL_AVREC_STOP = 0x3029
    CMD_LOCAL_PLAYBACK_START = 0x3030
    CMD_LOCAL_PLAYBACK_STOP = 0x3031
    CMD_LOCAL_PLAYBACK_SEEK = 0x3032
    CMD_LOCAL_PLAYBACK_PAUSE = 0x3033
    CMD_LOCAL_PLAYBACK_RESUME = 0x3034
    CMD_LOCAL_PLAYBACK_START1 = 0x3035
    CMD_LOCAL_PLAYBACK_STEP = 0x3036
    CMD_LOCAL_PLAYBACK_START2 = 0x3037
    CMD_LOCAL_MJREC_START = 0x3040
    CMD_LOCAL_MJREC_STOP = 0x3041
    ACK_PEER_LIVEAUDIO_START = 0x3100
    ACK_PEER_LIVEAUDIO_STOP = 0x3101
    ACK_LOCAL_LIVEAUDIO_START = 0x3102
    ACK_LOCAL_LIVEAUDIO_STOP = 0x3103
    ACK_PEER_AUDIOPARAM_SET = 0x3104
    ACK_PEER_AUDIOPARAM_GET = 0x3105
    ACK_PEER_AUDIOFILE_STARTPLAY = 0x3106
    ACK_PEER_AUDIOFILE_STOPPLAY = 0x3107
    ACK_PEER_AUDIOFILELIST_GET = 0x3108
    ACK_PEER_IRCUT_ONOFF = 0x310A
    ACK_PEER_LIGHTFILL_ONOFF = 0x310B
    ACK_PEER_LIVEVIDEO_START = 0x3110
    ACK_PEER_LIVEVIDEO_STOP = 0x3111
    ACK_PEER_PLAYBACK_START = 0x3112
    ACK_PEER_PLAYBACK_STOP = 0x3113
    ACK_PEER_PLAYBACK_SEEK = 0x3114
    ACK_PEER_PLAYBACK_SPEED = 0x3115
    ACK_PEER_PLAYBACK_PAUSE = 0x3116
    ACK_PEER_PLAYBACK_RESUME = 0x3117
    ACK_PEER_VIDEOPARAM_SET = 0x3118
    ACK_PEER_VIDEOPARAM_GET = 0x3119
    ACK_SNAPSHOT_GET = 0x311A
    ACK_PEER_PLAYBACK_END = 0x311B
    ACK_PEER_PLAYBACK_STEP = 0x311C
    CMD_FILE_CTRL = 0x4000
    CMD_FILETRANSFER_FILELIST_GET = 0x4005
    CMD_LOCALPATH = 0x4010
    ACK_FILE_CREATE = 0x4101
    ACK_FILE_RENAME = 0x4102
    ACK_FILE_DELETE = 0x4103
    ACK_FILE_MOVE = 0x4104
    ACK_FILE_LIST = 0x4105
    ACK_FILE_DOWNLOAD = 0x4110
    ACK_FILE_DOWNLOAD_PAUSE = 0x4111
    ACK_FILE_DOWNLOAD_RESUME = 0x4112
    ACK_FILE_DOWNLOAD_CANCEL = 0x4113
    ACK_FILE_UPLOAD = 0x4120
    ACK_FILE_UPLOAD_PAUSE = 0x4121
    ACK_FILE_UPLOAD_RESUME = 0x4122
    ACK_FILE_UPLOAD_CANCEL = 0x4123
    CMD_PTZ_SET = 0x5049
    ACK_PTZ_SET = 0x5149
    CMD_PTZ_GET = 0x5050
    CMD_PASSTHROUGH_STRING_PUT = 0x50FF
    ACK_PASSTHROUGH_STRING_PUT = 0x51FF
    CMD_SESSION_CHECK = 0x55FE
    CB_IEGET_STATUS = 0x6001
    CMD_NET_WIFISETTING_SET = 0x6001
    CB_IEGET_PARAM = 0x6002
    CMD_NET_WIFISETTING_GET = 0x6002
    CB_IEGET_CAM_PARAMS = 0x6003
    CMD_NET_WIFI_SCAN = 0x6003
    CB_IEGET_LOG = 0x6004
    CMD_NET_WIREDSETTING_SET = 0x6004
    CB_IEGET_MISC = 0x6005
    CMD_NET_WIREDSETTING_GET = 0x6005
    CB_IEGET_RECORD = 0x6006
    CB_IEGET_RECORD_FILE = 0x6007
    CB_IEGET_WIFI_SCAN = 0x6008
    CB_IEGET_FACTORY = 0x6009
    CB_IESET_IR = 0x600A
    CB_IESET_UPNP = 0x600B
    CB_IESET_ALARM = 0x600C
    CB_IESET_LOG = 0x600D
    CB_IESET_USER = 0x600E
    CB_IESET_ALIAS = 0x600F
    CB_IESET_MAIL = 0x6010
    CB_IESET_WIFI = 0x6011
    CB_CAM_CONTROL = 0x6012
    CB_IESET_DATE = 0x6013
    CB_IESET_MEDIA = 0x6014
    CB_IESET_SNAPSHOT = 0x6015
    CB_IESET_DDNS = 0x6016
    CB_IESET_MISC = 0x6017
    CB_IEGET_FTPTEST = 0x6018
    CB_DECODER_CONTROL = 0x6019
    CB_IESET_DEFAULT = 0x601A
    CB_IESET_MOTO = 0x601B
    CB_IEGET_MAILTEST = 0x601C
    CB_IESET_MAILTEST = 0x601D
    CB_IEDEL_FILE = 0x601E
    CB_IELOGIN = 0x601F
    CB_IESET_DEVICE = 0x6020
    CB_IESET_NETWORK = 0x6021
    CB_IESET_FTPTEST = 0x6022
    CB_IESET_DNS = 0x6023
    CB_IESET_OSD = 0x6024
    CB_IESET_FACTORY = 0x6025
    CB_IESET_PPPOE = 0x6026
    CB_IEREBOOT = 0x6027
    CB_IEFORMATSD = 0x6028
    CB_IESET_RECORDSCH = 0x6029
    CB_IESET_WIFISCAN = 0x602A
    CB_IERESTORE = 0x602B
    CB_IESET_FTP = 0x602C
    CB_IESET_RTSP = 0x602D
    CB_IEGET_VIDEOSTREAM = 0x602E
    CB_UPGRADE_APP = 0x602F
    CB_UPGRADE_SYS = 0x6030
    CB_SET_IIC = 0x6031
    CB_GET_IIC = 0x6032
    CB_IEGET_ALARMLOG = 0x6033
    CB_IESET_ALARMLOGCLR = 0x6034
    CB_IEGET_SYSWIFI = 0x6035
    CB_IESET_SYSWIFI = 0x6036
    CB_IEGET_LIVESTREAM = 0x6037
    CB_NOTIFICATION = 0x6040
    CB_IEGET_BILL = 0x6053
    CB_APP_VERSION = 0x6054
    CB_CHECK_USER = 0x60A0
    CB_IESET_BILL = 0x60A1
    ACK_NET_WIFISETTING_SET = 0x6101
    ACK_NET_WIFISETTING_GET = 0x6102
    ACK_NET_WIFI_SCAN = 0x6103
    ACK_NET_WIREDSETTING_SET = 0x6104
    ACK_NET_WIREDSETTING_GET = 0x6105
    CMD_FRIEND_MSG = 0x7000
    CB_SET_P2PPARAM = 0x99F0
    CB_GET_SYSOPR = 0x99FE
    CB_SET_SYSOPR = 0x99FF
    CMD_LOCAL_SESSION_INF = 0xF000
    CMD_LOCAL_SESSION_CHECK = 0xF001
    CMD_LOCAL_SESSION_GET = 0xF002
    CMD_LOCAL_SESSION_CTRL = 0xF003
    CMD_LOCAL_REC_START = 0xF004
    CMD_LOCAL_REC_STOP = 0xF005
    CMD_LOCAL_REC_MERGECTRL = 0xF006
    CMD_LOCAL_P2P_START = 0xF007
    CMD_LOCAL_P2P_STOP = 0xF008
    CMD_SESSION_CLOSE = 0xF00F
    CMD_LOCAL_PUSH_STRING = 0xF010
    CMD_LOCAL_PUSH_CFG = 0xF011
    CMD_LOCAL_RCVVID_DEC = 0xF012
    CMD_LOCAL_LAPSED = 0xF021
    CB_SET_SINGLE_SETTING_DEFAULT = 0xFF01
    CB_GET_FILE = 0xFF10
    CB_PUT_FILE = 0xFF11
    CB_SET_FILE = 0xFF12
    CB_GET_FILELIST = 0xFF13
    CB_SET_GPIO = 0xFF14
    CB_GET_GPIO = 0xFF15
    CB_GET_ADC = 0xFF16


CC_DEST = {
    BinaryCommands.CMD_SYSTEM_USER_CHK: 0x00ff,
    BinaryCommands.CMD_SYSTEM_STATUS_GET: 0x0000,
    BinaryCommands.CMD_PEER_LIVEVIDEO_START: 0x0000,
    BinaryCommands.CMD_PEER_LIVEVIDEO_STOP: 0x0000,  # ????
    BinaryCommands.CMD_NET_WIFI_SCAN: 0x0000,
    BinaryCommands.CMD_NET_WIFISETTING_GET: 0x0000,
    BinaryCommands.CMD_PASSTHROUGH_STRING_PUT: 0x0000,

    BinaryCommands.ACK_NET_WIFI_SCAN: 0x55aa,
    BinaryCommands.ACK_SYSTEM_USER_CHK: 0x00ff,  # 0x55aa,
    BinaryCommands.ACK_SYSTEM_STATUS_GET: 0x55aa,
}


class JsonCommands(Enum):
    CMD_SET_CYPUSH = 1
    CMD_CHECK_USER = 100
    CMD_GET_PARMS = 101
    CMD_DEV_CONTROL = 102
    CMD_EDIT_USER = 106
    CMD_GET_ALARM = 107
    CMD_SET_ALARM = 108
    CMD_STREAM = 111
    CMD_GET_WIFI = 112
    CMD_SCAN_WIFI = 113
    CMD_SET_WIFI = 114
    CMD_SET_DATETIME = 126  # returns result as cmd 128...
    CMD_PTZ_CONTROL = 128
    CMD_GET_RECORD_PARAM = 199
    CMD_TALK_SEND = 300
    CMD_SET_WHITELIGHT = 304
    CMD_GET_WHITELIGHT = 305
    CMD_GET_CLOUD_SUPPORT = 9000


class PTZ(Enum):
    # Pan-tilt-zoom control
    UP_START = 0
    UP_STOP = 1
    DOWN_START = 2
    DOWN_STOP = 3
    LEFT_START = 4
    LEFT_STOP = 5
    RIGHT_START = 6
    RIGHT_STOP = 7

    # TILT_UP_START = 0
    # TILT_UP_STOP = 1
    # TILT_DOWN_START = 2
    # TILT_DOWN_STOP = 3
    # PAN_LEFT_START = 4
    # PAN_LEFT_STOP = 5
    # PAN_RIGHT_START = 6
    # PAN_RIGHT_STOP = 7


JSON_COMMAND_NAMES = {
    JsonCommands.CMD_SET_CYPUSH: "set_cypush",
    JsonCommands.CMD_CHECK_USER: "check_user",
    JsonCommands.CMD_GET_PARMS: "get_parms",
    JsonCommands.CMD_DEV_CONTROL: "dev_control",
    JsonCommands.CMD_EDIT_USER: "edit_user",
    JsonCommands.CMD_GET_ALARM: "get_alarm",
    JsonCommands.CMD_SET_ALARM: "set_alarm",
    JsonCommands.CMD_STREAM: "stream",
    JsonCommands.CMD_GET_WIFI: "get_wifi",
    JsonCommands.CMD_SCAN_WIFI: "scan_wifi",
    JsonCommands.CMD_SET_WIFI: "set_wifi",
    JsonCommands.CMD_SET_DATETIME: "set_datetime",
    JsonCommands.CMD_PTZ_CONTROL: "ptz_control",
    JsonCommands.CMD_GET_RECORD_PARAM: "get_record_param",
    JsonCommands.CMD_TALK_SEND: "talk_send",
    JsonCommands.CMD_SET_WHITELIGHT: "set_whiteLight",
    JsonCommands.CMD_GET_WHITELIGHT: "get_whiteLight",
    JsonCommands.CMD_GET_CLOUD_SUPPORT: "get_cloudsupport",
}

class PtzDirection(IntEnum):
    PTZ_DIRECTION_UP = 0
    PTZ_DIRECTION_DOWN = 1
    PTZ_DIRECTION_LEFT = 2
    PTZ_DIRECTION_RIGHT = 3

    PTZ_DIRECTION_UP_DOWN = 4
    PTZ_DIRECTION_LEFT_RIGHT = 5
    PTZ_DIRECTION_LEFT_UP = 6
    PTZ_DIRECTION_LEFT_DOWN = 7
    PTZ_DIRECTION_RIGHT_UP = 8
    PTZ_DIRECTION_RIGHT_DOWN = 9

    PTZ_DIRECTION_CENTER = 10

    PTZ_DIRECTION_ORIGINAL = 11
    PTZ_DIRECTION_STOP = 12

    PTZ_DIRECTION_PRE_TO = 13
    PTZ_DIRECTION_PRE_REC = 14

    PTZ_DIRECTION_SPEED = 15

class PtzParamType(IntEnum):
    PTZ_PARAM_TYPE_DIRECTION = 0
    PTZ_PARAM_TYPE_PREFAB = 1
    PTZ_PARAM_TYPE_CRNTPOS = 2

class PtzPrefab(IntEnum):
    PTZ_PREFAB_SET = 0
    PTZ_PREFAB_GET = 1
    PTZ_PREFAB_REC = 2
    PTZ_PREFAB_CHK = 3
    PTZ_PREFAB_DEL = 4
    PTZ_PREFAB_INT = 5

# Seems that only Resolution and Bitrate are supported
class VideoParamType(IntEnum):
    VIDEO_PARAM_TYPE_DEFAULT = 0
    VIDEO_PARAM_TYPE_RESOLUTION = 1
    VIDEO_PARAM_TYPE_BRIGHTNESS = 2
    VIDEO_PARAM_TYPE_CONTRAST = 3
    VIDEO_PARAM_TYPE_SATURATION = 4
    VIDEO_PARAM_TYPE_SHARPNESS = 5
    VIDEO_PARAM_TYPE_FRAMERATE = 6
    VIDEO_PARAM_TYPE_BITRATE = 7
    VIDEO_PARAM_TYPE_ROTATE = 8
    VIDEO_PARAM_TYPE_IRCUT = 9
    VIDEO_PARAM_TYPE_OSD = 10
    VIDEO_PARAM_TYPE_MOVEDETECTION = 11
    VIDEO_PARAM_TYPE_MODE = 12
    VIDEO_PARAM_TYPE_SCENE_NORMAL = 90
    VIDEO_PARAM_TYPE_SCENE_HUMAN = 91
    VIDEO_PARAM_TYPE_SCENE_COOL = 92
    VIDEO_PARAM_TYPE_SCENE_BLACKLIGHT = 93

# HD and up may not be supported on most devices
class VideoResolution(IntEnum):
    VIDEO_RESOLUTION_QVGA = 0
    VIDEO_RESOLUTION_VGA = 1
    VIDEO_RESOLUTION_HD = 2
    VIDEO_RESOLUTION_FD = 3
    VIDEO_RESOLUTION_UD = 4

class VideoRotate(IntEnum):
    VIDEO_ROTATE_NORMAL = 0
    VIDEO_ROTATE_H = 1
    VIDEO_ROTATE_V = 2
    VIDEO_ROTATE_HV = 3